<?php
/* -------------------------------------------
 * MobileCMS
 * ------------------------------------------- */
uses('folder');

class AddonsController extends AppController {

	var $name = 'Addons';
	var $helpers = array('Html','Javascript','Zarame');
	var $components = array('context');


	function admin_index() {
		// Title of page
		//$this->pageTitle = $this->siteName.' - Addons Manager!';
		// Create new folder object
		$folderObj = new Folder(APP.'plugins');
		// Find addon_description.php in each add-on and return it
		$configFiles = $folderObj->findRecursive('addon_description.php');
		// Initialize variable
		$uninstalledAddons = array();
		// Loop through configuration files
		foreach ($configFiles as $configFile) {
			// Explode configuration file by directory separator
			$configFileArr = explode(DS, $configFile);
			// Get add-on directory name
			$dirName = $configFileArr[count($configFileArr) - 2];
			// Check that this add-on is in database or not
			if (!is_array($this->Addon->findByDirectory($dirName))) {
				// Initialize variables
				$name = $version = $description = '';
				// Include add-on's configuration file
				include $configFile;
				// Store un-installed add-on's information
				$uninstalledAddons[] = array(
										'name' => $name,
										'version' => $version,
										'directory' => $dirName,
										'description' => $description
										);
			}
			// Unset variable
			unset($configFileArr);
		}
		// Get add-ons from database
		$installedAddons = $this->Addon->findAll();
		// Loop through add-ons
		foreach ($installedAddons as $k => $addon) {
			// Initialize variables
			$name = $version = $description = '';
			// Include add-on's configuration file
			include APP.'plugins'.DS.$addon['Addon']['directory'].DS.'addon_description.php';
			// Store installed add-on's information
			$installedAddons[$k]['Addon']['name'] = $name;
			$installedAddons[$k]['Addon']['version'] = $version;
			$installedAddons[$k]['Addon']['description'] = $description;
			// If add-on is activated then load it's manager GUI
			if ($addon['Addon']['activated'] == 'yes'){
				$installedAddons[$k]['Addon']['gui'] = trim($this->requestAction('/admin/'.$addon['Addon']['directory'].'/managerGui', array('return')));
			} else {
				$installedAddons[$k]['Addon']['gui'] = '';
			}
		}
		// Set variables to use in view
		$this->set('installedAddons', $installedAddons);
		$this->set('uninstalledAddons', $uninstalledAddons);
	}


	function admin_activate($id = null) {
		// No need to render view for this action
		$this->autoRender = false;
		// Type cast add-on id as integer
		$id = (int)$id;
		// Get add-on's details
		$addonDetails = $this->Addon->findById($id);
		// Check if add-on exists in database or not
		if (is_array($addonDetails)) {
			// Populate data to be add to database table
			$this->data = array('Addon' => array('id' => $id, 'activated' => 'yes'));
			$this->params['data'] =& $this->data;
			// Update database table
			if ($this->Addon->save($this->params['data'])) {
				$this->Session->del('loadedAddons');
				$this->Session->setFlash("Add-on '".$addonDetails['Addon']['directory']."' has been activated successfully.",'default',array(),'success');
			} else {
				$this->Session->setFlash("Unable to activate add-on '".$addonDetails['Addon']['directory']."' into database table.",'default',array(),'error');
			}
		} else {
			$this->Session->setFlash("Unable to activate add-on. Invalid add-on id '$id'.",'default',array(),'error');
		}
		// Redirect to add-on manager
		$this->redirect('/admin/addons');
	}


	function admin_deactivate($id = null) {
		// No need to render view for this action
		$this->autoRender = false;
		// Type cast add-on id as integer
		$id = (int)$id;
		// Get add-on's details
		$addonDetails = $this->Addon->findById($id);
		// Check if add-on exists in database or not
		if (is_array($addonDetails)) {
			// Populate data to be add to database table
			$this->data = array('Addon' => array('id' => $id, 'activated' => 'no'));
			$this->params['data'] =& $this->data;
			// Update database table
			if ($this->Addon->save($this->params['data'])) {
				$this->Session->del('loadedAddons');
				$this->Session->setFlash("Add-on '".$addonDetails['Addon']['directory']."' has been de-activated successfully.",'default',array(),'success');
			} else {
				$this->Session->setFlash("Unable to de-activate add-on '".$addonDetails['Addon']['directory']."' into database table.",'default',array(),'error');
			}
		} else {
			$this->Session->setFlash("Unable to de-activate add-on. Invalid add-on id '$id'.",'default',array(),'error');
		}
		// Redirect to add-on manager
		$this->redirect('/admin/addons');
	}


	function admin_install($addonDirectory = null) {
		// No need to render view for this action
		$this->autoRender = false;
		// Create new sanitize object
		$sanitize = new Sanitize;
		// Sanitize add-on directory's name
		$addonDirectory = $sanitize->paranoid($addonDirectory);
		// Check if add-on directory exists or not
		if (is_dir(APP.'plugins'.DS.$addonDirectory)) {
			// Check if add-on already installed or not
			if (is_array($this->Addon->findByDirectory($addonDirectory))) {
				$this->Session->setFlash("Add-on '$addonDirectory' already installed.",'default',array(),'error');
			} else {
				// Check if addon_install.php exists or not
				if (is_file(APP.'plugins'.DS.$addonDirectory.DS.'addon_install.php')) {
					// Include addon_install.php for add-on
					include(APP.'plugins'.DS.$addonDirectory.DS.'addon_install.php');
					// Check if some pre-installation database queries exists or not.
					// If exists then check that it should be non-empty array
					if (isset($installQueries) && is_array($installQueries) && count($installQueries) > 0) {
						// Loop through pre-installation database queries
						foreach ($installQueries as $query) {
							// Run database query
							$this->Addon->query($query);
						}
						// Clear cache so that new database table created by add-on's install will be stored in cache
						clearCache(null, 'models', '');
						// Get database object
						$db =& ConnectionManager::getDataSource($this->Addon->useDbConfig);
						// This is a dirty hack to force DB to re-read table list
						$db->__sources = null;
					}
				}
				// Call add-on's install action and get it's response
				$response = trim($this->requestAction('/admin/'.$addonDirectory.'/install', array('return')));
				// If add-on's install method runs successfully
				if ($response == 'true') {
					// Populate data to be add to database
					$this->data = array('Addon' => array('id' => 0, 'directory' => $addonDirectory, 'activated' => 'yes'));
					$this->params['data'] =& $this->data;
					// Store add-on details in database table
					if ($this->Addon->save($this->params['data'])) {
						$this->Session->del('loadedAddons');
						$this->Session->setFlash("Add-on '$addonDirectory' has been installed successfully.",'default',array(),'success');
					} else {
						$this->Session->setFlash("Unable to insert add-on '$addonDirectory' into database table.",'default',array(),'error');
					}
					// Unset variables
					unset($this->data);
					unset($this->params['data']);
				} else {
					$this->Session->setFlash("Unable to install add-on '$addonDirectory'. Error :- $response.",'default',array(),'error');
				}
			}
		} else {
			$this->Session->setFlash("Unable to install add-on. Invalid add-on directory '$addonDirectory'.",'default',array(),'error');
		}
		// Redirect to add-on manager
		$this->redirect('/admin/addons');
	}


	function admin_uninstall($id = null) {
		// No need to render view for this action
		$this->autoRender = false;
		// Type cast add-on id as integer
		$id = (int)$id;
		// Get add-on's details
		$addonDetails = $this->Addon->findById($id);
		// Check if add-on exists in database or not
		if (is_array($addonDetails)) {
			// Call add-on's un-install action and get it's response
			$response = trim($this->requestAction('/admin/'.$addonDetails['Addon']['directory'].'/uninstall', array('return')));
			// If add-on's un-install method runs successfully
			if ($response == 'true') {
				// Check if addon_uninstall.php exists or not
				if (is_file(APP.'plugins'.DS.$addonDetails['Addon']['directory'].DS.'addon_uninstall.php')) {
					// Include addon_uninstall.php for add-on
					include(APP.'plugins'.DS.$addonDetails['Addon']['directory'].DS.'addon_uninstall.php');
					// Check if some pre-uninstallation database queries exists or not.
					// If exists then check that it should be non-empty array
					if (isset($uninstallQueries) && is_array($uninstallQueries) && count($uninstallQueries) > 0) {
						// Loop through pre-uninstallation database queries
						foreach ($uninstallQueries as $query) {
							// Run database query
							$this->Addon->query($query);
						}
					}
				}
				// Remove add-on details from database table
				if ($this->Addon->del($id)) {
					$this->Session->del('loadedAddons');
					$this->Session->setFlash("Add-on '".$addonDetails['Addon']['directory']."' has been un-installed successfully.",'default',array(),'success');
				} else {
					$this->Session->setFlash("Unable to delete add-on '$addonDirectory' from database table.",'default',array(),'error');
				}
			} else {
				$this->Session->setFlash("Unable to un-install add-on '".$addonDetails['Addon']['directory']."'. Error :- $response.",'default',array(),'error');
			}
		} else {
			$this->Session->setFlash("Unable to un-install add-on. Invalid add-on id '$id'.",'default',array(),'error');
		}
		// Redirect to add-on manager
		$this->redirect('/admin/addons');
	}


	// Added empty method here so that redirect in app controller won't execute
	// any code in app controller's afterFilter method
	function afterFilter()
	{
	}
}